@@ -4,7 +4,7 @@
 const {
     Atk, Clutter, Gio, GLib, GMenu, GObject, Gtk, Meta, Shell, St,
 } = imports.gi;
-const Signals = imports.signals;
+const {EventEmitter} = imports.misc.signals;
 
 const DND = imports.ui.dnd;
 const ExtensionUtils = imports.misc.extensionUtils;
@@ -12,8 +12,7 @@ const Main = imports.ui.main;
 const PanelMenu = imports.ui.panelMenu;
 const PopupMenu = imports.ui.popupMenu;
 
-const Gettext = imports.gettext.domain('gnome-shell-extensions');
-const _ = Gettext.gettext;
+const _ = ExtensionUtils.gettext;
 
 const appSys = Shell.AppSystem.get_default();
 
@@ -25,10 +24,13 @@ const NAVIGATION_REGION_OVERSHOOT = 50;
 Gio._promisify(Gio._LocalFilePrototype, 'query_info_async', 'query_info_finish');
 Gio._promisify(Gio._LocalFilePrototype, 'set_attributes_async', 'set_attributes_finish');
 
-var ApplicationMenuItem = GObject.registerClass(
 class ApplicationMenuItem extends PopupMenu.PopupBaseMenuItem {
-    _init(button, app) {
-        super._init();
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(button, app) {
+        super();
         this._app = app;
         this._button = button;
 
@@ -94,12 +96,15 @@ class ApplicationMenuItem extends PopupMenu.PopupBaseMenuItem {
         icon.style_class = 'icon-dropshadow';
         this._iconBin.set_child(icon);
     }
-});
+}
 
-var CategoryMenuItem = GObject.registerClass(
 class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
-    _init(button, category) {
-        super._init();
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(button, category) {
+        super();
         this._category = category;
         this._button = button;
 
@@ -112,7 +117,7 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         else
             name = _('Favorites');
 
-        this.add_child(new St.Label({ text: name }));
+        this.add_child(new St.Label({text: name}));
         this.connect('motion-event', this._onMotionEvent.bind(this));
         this.connect('notify::active', this._onActiveChanged.bind(this));
     }
@@ -181,11 +186,14 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
     }
 
     _onMotionEvent(actor, event) {
-        let device = event.get_device();
-        if (!device.get_grabbed_actor()) {
+        if (!this._grab) {
             this._oldX = -1;
             this._oldY = -1;
-            device.grab(this);
+            const grab = global.stage.grab(this);
+            if (grab.get_seat_state() !== Clutter.GrabState.NONE)
+                this._grab = grab;
+            else
+                grab.dismiss();
         }
         this.hover = true;
 
@@ -195,7 +203,8 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         this._oldX = -1;
         this._oldY = -1;
         this.hover = false;
-        device.ungrab();
+        this._grab?.dismiss();
+        delete this._grab;
 
         let source = event.get_source();
         if (source instanceof St.Widget)
@@ -211,7 +220,7 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         this._button.selectCategory(this._category);
         this._button.scrollToCatButton(this);
     }
-});
+}
 
 class ApplicationsMenu extends PopupMenu.PopupMenu {
     constructor(sourceActor, arrowAlignment, arrowSide, button) {
@@ -230,8 +239,10 @@ class ApplicationsMenu extends PopupMenu.PopupMenu {
     }
 }
 
-class DesktopTarget {
+class DesktopTarget extends EventEmitter {
     constructor() {
+        super();
+
         this._desktop = null;
         this._desktopDestroyedId = 0;
 
@@ -348,12 +359,14 @@ class DesktopTarget {
         return true;
     }
 }
-Signals.addSignalMethods(DesktopTarget.prototype);
 
-let ApplicationsButton = GObject.registerClass(
 class ApplicationsButton extends PanelMenu.Button {
-    _init() {
-        super._init(1.0, null, false);
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor() {
+        super(1.0, null, false);
 
         this.setMenu(new ApplicationsMenu(this, 1.0, St.Side.TOP, this));
         Main.panel.menuManager.addMenu(this.menu);
@@ -379,9 +392,12 @@ class ApplicationsButton extends PanelMenu.Button {
         this._hidingId = Main.overview.connect('hiding', () => {
             this.remove_accessible_state(Atk.StateType.CHECKED);
         });
-        Main.layoutManager.connect('startup-complete',
-            this._setKeybinding.bind(this));
-        this._setKeybinding();
+        Main.wm.addKeybinding(
+            'apps-menu-toggle-menu',
+            ExtensionUtils.getSettings(),
+            Meta.KeyBindingFlags.IGNORE_AUTOREPEAT,
+            Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW,
+            () => this.menu.toggle());
 
         this._desktopTarget = new DesktopTarget();
         this._desktopTarget.connect('app-dropped', () => {
@@ -393,7 +409,7 @@ class ApplicationsButton extends PanelMenu.Button {
             });
         });
 
-        this._tree = new GMenu.Tree({ menu_basename: 'applications.menu' });
+        this._tree = new GMenu.Tree({menu_basename: 'applications.menu'});
         this._treeChangedId = this._tree.connect('changed',
             this._onTreeChanged.bind(this));
 
@@ -432,11 +448,7 @@ class ApplicationsButton extends PanelMenu.Button {
         this._tree.disconnect(this._treeChangedId);
         this._tree = null;
 
-        Main.wm.setCustomKeybindingHandler('panel-main-menu',
-            Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW,
-            Main.sessionMode.hasOverview
-                ? Main.overview.toggle.bind(Main.overview)
-                : null);
+        Main.wm.removeKeybinding('apps-menu-toggle-menu');
 
         this._desktopTarget.destroy();
     }
@@ -478,12 +490,6 @@ class ApplicationsButton extends PanelMenu.Button {
         super._onOpenStateChanged(menu, open);
     }
 
-    _setKeybinding() {
-        Main.wm.setCustomKeybindingHandler('panel-main-menu',
-            Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW,
-            () => this.menu.toggle());
-    }
-
     _redisplay() {
         this.applicationsBox.destroy_all_children();
         this.categoriesBox.destroy_all_children();
@@ -504,7 +510,7 @@ class ApplicationsButton extends PanelMenu.Button {
                 }
                 let app = appSys.lookup_app(id);
                 if (!app)
-                    app = new Shell.App({ app_info: entry.get_app_info() });
+                    app = new Shell.App({app_info: entry.get_app_info()});
                 if (app.get_app_info().should_show())
                     this.applicationsByCategory[categoryId].push(app);
             } else if (nextType === GMenu.TreeItemType.SEPARATOR) {
@@ -550,8 +556,8 @@ class ApplicationsButton extends PanelMenu.Button {
     _createLayout() {
         let section = new PopupMenu.PopupMenuSection();
         this.menu.addMenuItem(section);
-        this.mainBox = new St.BoxLayout({ vertical: false });
-        this.leftBox = new St.BoxLayout({ vertical: true });
+        this.mainBox = new St.BoxLayout({vertical: false});
+        this.leftBox = new St.BoxLayout({vertical: true});
         this.applicationsScrollBox = new St.ScrollView({
             style_class: 'apps-menu vfade',
             x_expand: true,
@@ -573,9 +579,9 @@ class ApplicationsButton extends PanelMenu.Button {
         vscroll.connect('scroll-stop', () => (this.menu.passEvents = false));
         this.leftBox.add_child(this.categoriesScrollBox);
 
-        this.applicationsBox = new St.BoxLayout({ vertical: true });
+        this.applicationsBox = new St.BoxLayout({vertical: true});
         this.applicationsScrollBox.add_actor(this.applicationsBox);
-        this.categoriesBox = new St.BoxLayout({ vertical: true });
+        this.categoriesBox = new St.BoxLayout({vertical: true});
         this.categoriesScrollBox.add_actor(this.categoriesBox);
 
         this.mainBox.add(this.leftBox);
@@ -669,21 +675,24 @@ class ApplicationsButton extends PanelMenu.Button {
 
         return applist;
     }
-});
+}
 
 let appsMenuButton;
 
+/** */
 function enable() {
     appsMenuButton = new ApplicationsButton();
     let index = Main.sessionMode.panel.left.indexOf('activities') + 1;
     Main.panel.addToStatusArea('apps-menu', appsMenuButton, index, 'left');
 }
 
+/** */
 function disable() {
     Main.panel.menuManager.removeMenu(appsMenuButton.menu);
     appsMenuButton.destroy();
 }
 
+/** */
 function init() {
     ExtensionUtils.initTranslations();
 }
